#@IgnoreInspection BashAddShebang
# Config file functionality (Koalephant Shell Script Library)
# Version: PACKAGE_VERSION
# Copyright: 2017, Koalephant Co., Ltd
# Author: Stephen Reay <stephen@koalephant.com>, Koalephant Packaging Team <packages@koalephant.com>

# Path to cfget binary
readonly KOALEPHANT_CFGET_PATH="CFGET_BIN"


# Run a cfget command
#
# Input:
# $1 - the config file
# $2...n - extra arguments for cfget
#
# Output:
# The result from cfget
#
# Return:
# The cfget return code
#
# Deprecated
# As of v2.5.0, use (#k_config_op) instead
k_config_command() {
	# shellcheck disable=SC2039
	local file

	k_requires_args k_config_command $#
	file="$1"
	shift

	k_log_deprecated k_config_command 2.5.0 'k_config_op() or k_config_op_cfget()'

	"${KOALEPHANT_CFGET_PATH}" --plugin "${KOALEPHANT_LIB_PATH}/cfget-plugins/" --format nestedini --cfg "${file}" --quiet "$@"
}

# Perform a config file operation
#
# Input:
# $1 - the operation to perform, one of 'sections', 'keys', and 'read'
# $2 - the config file
# $3...n - extra arguments for the operation
# $KOALEPHANT_CONFIG_MODULE an override to use a specific config module
#
# Output:
# The result from the config module
#
# Return:
# 0 unless attempting to read a non-existent key, then 1, or no config module is usable, then 2.
k_config_op() {

	override_module() {
		[ "${KOALEPHANT_CONFIG_MODULE:-${1}}" = "${1}" ]
	}

	if k_config_op_php avail "$@" && override_module php; then
		k_config_op_php "$@"
	elif k_config_op_python2 avail "$@" && override_module python2; then
		k_config_op_python2 "$@"
	elif k_config_op_python3 avail "$@" && override_module python3; then
		k_config_op_python3 "$@"
	elif k_config_op_php_ro avail "$@" && override_module php_ro; then
		k_config_op_php_ro "$@"
	else
		k_log_err 'No config module available'
		return 2
	fi
}


# Get config sections
#
# Input:
# $1 - the config file
# $2 - the config root to work from, defaults to empty
#
# Output:
# the section names, one per line
k_config_sections() {
	k_requires_args k_config_sections $#

	k_config_op sections "$@"
}

# Get config keys
#
# Input:
# $1 - the config file
# $2 - the config root to work from
#
# Output:
# the key names, one per line
k_config_keys() {
	k_requires_args k_config_keys $#

	k_config_op keys "$@"
}

# Read a Config Value
# Unlike (#k_config_read_error), this function will not error if the config key is not found
#
# Input:
# $1 - the config file
# $2 - the key to read
# $3 - the config root to work from
#
# Output:
# the value of the key, if it exists
k_config_read() {
	k_config_read_error "$@" || true
}

# Read a Config Value, or error
#
# Input:
# $1 - the config file
# $2 - the key to read
# $3 - the config root to work from
#
# Output:
# the value of the key, if it exists
#
# Return:
# 0 on success, 1 on error
k_config_read_error() {
	k_requires_args k_config_read_error $# 2

	k_config_op read "$@"
}

# Get a Config value or a default value
#
# Input:
# $1 - the config file
# $2 - the key to read
# $3 - the default value (defaults to empty string)
# $4 - the config root to work from
#
# Output:
# the value of the key, or the default value
k_config_read_string() {
	# shellcheck disable=SC2039
	local file key root default value

	k_requires_args k_config_read_string $# 2

	file="$1"
	key="$2"
	default="${3:-}"
	root="${4:-}"
	value="$(k_config_read "$file" "$key" "${root}")"

	printf -- '%s\n' "${value:-${default}}"
}

# Get a 'Boolean' Config value or a default value
#
# Input:
# $1 - the config file
# $2 - the key to read
# $3 - the default value (defaults to empty string)
# $4 - the config root to work from
#
# Output:
# `true`, `false` or the default value given if the config value cannot be parsed using (bool.lib#k_bool_parse)
k_config_read_bool() {
	# shellcheck disable=SC2039
	local file key root default

	k_requires_args k_config_read_bool $# 2

	file="$1"
	key="$2"
	default="${3:-}"
	root="${4:-}"

	k_bool_parse "$(k_config_read "${file}" "${key}" "${root}")" "${default}"
}

# Test a 'Boolean' Config value or a default value
#
# Input:
# $1 - the config file
# $2 - the key to read
# $3 - the default value (defaults to empty string)
# $4 - the config root to work from
#
# Return:
# 0 if either the config value or the default value is true, 1 otherwise
k_config_test_bool() {
	# shellcheck disable=SC2039
	local file key root default

	k_requires_args k_config_test_bool $# 2

	file="$1"
	key="$2"
	default="${3:-}"
	root="${4:-}"

	k_bool_test "$(k_config_read_string "${file}" "${key}" "$(k_bool_parse "${default}")" "${root}")"
}

# Get a 'keyword' Config value or a default value
#
# Input:
# $1 - the config file
# $2 - the key to read
# $3 - the default value
# $4 - the config root to work from
# $5...n - the list of valid keyword values
#
# Output:
# the valid keyword or default value
k_config_read_keyword() {
	# shellcheck disable=SC2039
	local file key root default

	k_requires_args k_config_read_keyword $# 2

	file="$1"
	key="$2"
	default="${3:-}"
	root="${4:-}"

	shift 4

	k_string_keyword "$(k_config_read_string "${file}" "${key}" "${default}" "${root}")" "$@" "$default"
}

# Write a Config Value, or error
#
# Input:
# $1 - the config file
# $2 - the key to write to
# $3 - the value to write
# $4 - the config root to work from
#
# Return:
# 0 on success, 1 on error
k_config_write_error() {
	k_requires_args k_config_write_error $# 3
	k_config_op write "$@"
}


# Check that a readable file has been provided
#
# Input:
# $1 - the file pathname to try to read
#
# Return:
# 0 if the file pathname is readable, 1 if not
k_config_file_check_readable() {
	# shellcheck disable=SC2039
	local file="${1:-}"

	if [ -z "${file}" ]; then
		k_log_err 'No config file specified for reading'
		return 1
	fi

	if [ ! -r "${file}" ]; then
		k_log_err 'Cannot open config file "%s" for reading' "${file}"
		return 1
	fi
}

# Check that a writable file has been provided
#
# Input:
# $1 - the file pathname to try to read
#
# Output:
# the file pathname, if readable.
#
# Return:
# 0 if the file pathname is readable, 1 if not
k_config_file_check_writable() {
	# shellcheck disable=SC2039
	local file="${1:-}"

	if [ -z "${file}" ]; then
		k_log_err 'No config file specified for writing'
		return 1
	fi

	if [ ! -f "${file}" ]; then
		touch "${file}" 2>/dev/null || true
	fi

	if [ ! -r "${file}" ] || [ ! -w "${file}" ]; then
		k_log_err 'Cannot open config file "%s" for writing' "${file}"
		return 1
	fi
}


# PHP based Config Module
#
# Input:
# $1 - the operation to perform, one of 'avail', 'sections', 'keys', and 'read'
# $2 - the file to operate on
# $3...n - any additional operation-specific arguments
#
# Output:
# the result from the php script
#
# Return:
# 0 or 1 for success or failure, depending on operation being performed
k_config_op_php() {
	# shellcheck disable=SC2039
	local op file php="${KOALEPHANT_CONFIG_PHP:-php}"
	k_requires_args k_config_op_php $# 2

	op="$1"
	shift

	case "${op}" in
		(avail)
			if command -v "${php}" >/dev/null && "${php}" "${KOALEPHANT_LIB_PATH}/config-modules/config-dba.php" "${op}"; then
				return 0
			fi
			return 1
		;;

		(whoami)
			"${php}" --version
		;;

		(sections|keys|read)
			if ! k_config_file_check_readable "$@"; then
				return 1
			fi

			"${php}" "${KOALEPHANT_LIB_PATH}/config-modules/config-dba.php" "${op}" "$@"
		;;

		(write)
			k_requires_args k_config_op_php $(( $#+1 )) 3

			if ! k_config_file_check_writable "$@"; then
				return 1
			fi

			"${php}" "${KOALEPHANT_LIB_PATH}/config-modules/config-dba.php" "${op}" "$@"
		;;


		(*)
			false
		;;

	esac
}


# PHP (readonly) Config Module
#
# Input:
# $1 - the operation to perform, one of 'avail', 'sections', 'keys', and 'read'
# $2 - the file to operate on
# $3...n - any additional operation-specific arguments
#
# Output:
# the result from the php script
#
# Return:
# 0 or 1 for success or failure, depending on operation being performed
k_config_op_php_ro() {
	# shellcheck disable=SC2039
	local op realOp file php="${KOALEPHANT_CONFIG_PHP:-php}"
	k_requires_args k_config_op_php $# 2

	op="$1"
	shift

	case "${op}" in
		(avail)
			if command -v "${php}" >/dev/null; then
				realOp="$1"
				case "${realOp}" in
					(avail|whoami|sections|keys|read)
						return 0
					;;
				esac
			fi
			return 1
		;;

		(whoami)
			"${php}" --version
		;;

		(sections|keys|read)
			if ! k_config_file_check_readable "$@"; then
				return 1
			fi

			"${php}" "${KOALEPHANT_LIB_PATH}/config-modules/config-ro.php" "${op}" "$@"
		;;

		(*)
			false
		;;

	esac
}

# Python2 based Config Module
#
# Input:
# $1 - the operation to perform, one of 'avail', 'sections', 'keys', and 'read'
# $2 - the file to operate on
# $3...n - any additional operation-specific arguments
#
# Output:
# the result from the php script
#
# Return:
# 0 or 1 for success or failure, depending on operation being performed
k_config_op_python2() {
	# shellcheck disable=SC2039
	local op file python2="${KOALEPHANT_CONFIG_PYTHON2:-python}" foundVersion=''
	k_requires_args k_config_op_python2 $# 2

	op="$1"
	shift

	case "${op}" in
		(avail)
			command -v "${python2}" >/dev/null
			foundVersion="$("${python2}" --version 2>&1 | cut -f 2 -d ' ')"
			k_version_compare "${foundVersion}" 2 '>=' && k_version_compare "${foundVersion}" 3 '<'
		;;

		(whoami)
			"${python2}" --version 2>&1
		;;

		(sections|keys|read)
			if ! k_config_file_check_readable "$@"; then
				return 1
			fi

			"${python2}" "${KOALEPHANT_LIB_PATH}/config-modules/config2.py" "${op}" "$@"
		;;

		(write)
			k_requires_args k_config_op_php $(( $#+1 )) 3

			if ! k_config_file_check_writable "$@"; then
				return 1
			fi

			"${python2}" "${KOALEPHANT_LIB_PATH}/config-modules/config2.py" "${op}" "$@"
		;;

		(*)
			false
		;;
	esac
}

# Python3 based Config Module
#
# Input:
# $1 - the operation to perform, one of 'avail', 'sections', 'keys', and 'read'
# $2 - the file to operate on
# $3...n - any additional operation-specific arguments
#
# Output:
# the result from the php script
#
# Return:
# 0 or 1 for success or failure, depending on operation being performed
k_config_op_python3() {
	# shellcheck disable=SC2039
	local op file python3="${KOALEPHANT_CONFIG_PYTHON3:-python3}"
	k_requires_args k_config_op_python3 $# 2

	op="$1"
	shift

	case "${op}" in
		(avail)
			command -v "${python3}" >/dev/null
		;;

		(whoami)
			"${python3}" --version
		;;

		(sections|keys|read)
			if ! k_config_file_check_readable "$@"; then
				return 1
			fi

			"${python3}" "${KOALEPHANT_LIB_PATH}/config-modules/config3.py" "${op}" "$@"
		;;

		(write)
			k_requires_args k_config_op_php $(( $#+1 )) 3

			if ! k_config_file_check_writable "$@"; then
				return 1
			fi

			"${python3}" "${KOALEPHANT_LIB_PATH}/config-modules/config3.py" "${op}" "$@"
		;;

		(*)
			false
		;;
	esac
}
