#@IgnoreInspection BashAddShebang
# Filesystem functionality (Koalephant Shell Script Library)
# Version: PACKAGE_VERSION
# Copyright: 2017, Koalephant Co., Ltd
# Author: Stephen Reay <stephen@koalephant.com>, Koalephant Packaging Team <packages@koalephant.com>

readonly K_FS_TEMP_FILES_FILENAME='k-fs-temp-files'

# Resolve a given filesystem path to an absolute path
#
# Input:
# $1 - the path to resolve
#
# Output:
# the resolved path
k_fs_resolve() {
	# shellcheck disable=SC2039
	local path dir='' base=''

	k_requires_args k_fs_resolve $#

	path="${1%/}"

	if [ -d "${path}" ]; then
		dir="${path}"
	else
		dir="$(k_fs_dirname "${path}")"
		base="$(k_fs_basename "${path}")"
	fi

	if [ -z "${dir}" ]; then
		dir="."
	fi

	dir="$(unset CDPATH && cd "${dir}" && pwd -P)"
	dir="${dir%/}"

	if [ -L "${dir}/${base}" ]; then
		# shellcheck disable=SC2012
		base="$(unset CDPATH && cd "${dir}" && readlink "${base}" || printf -- '%s' "${base}")"
	fi

	printf -- '%s/%s'  "${dir}" "${base}"
}

# Get the base name of a filesystem path, optionally with an extension removed
#
# Input:
# $1 - the path to get the basename of
# $2 - the extension to remove if found
#
# Output:
# the base name of the given path
k_fs_basename() {
	# shellcheck disable=SC2039
	local path extension

	k_requires_args k_fs_basename $#

	path="${1%/}"
	extension="${2:-}"

	path="${path##*/}"

	if [ -n "${extension}" ]; then
		path="${path%$extension}"
	fi

	printf -- '%s\n' "${path:-/}"
}

# Get the extension of a filesystem path
#
# Input:
# $1 - the path to get the extension of
# $2 - flag to get all extension parts, rather than just the last part. Defaults to false.
#
# Output:
# the extension of the given path, if it has one
#
# Example:
# k_fs_extension foo.bar.baz [false] // baz
# k_fs_extension foo.bar.baz true // bar.baz
k_fs_extension() {
	# shellcheck disable=SC2039
	local path extension

	k_requires_args k_fs_extension $#

	path="$(k_fs_basename "${1}")"

	if [ "${2:-false}" = true ]; then
		extension="${path#*.}"
	else
		extension="${path##*.}"
	fi

	if [ "${extension}" != "${path}" ]; then
		printf -- '%s\n' "${extension}"
	fi
}


# Get the parent directory path of a filesystem path
#
# Input:
# $1 - the path to get the parent directory path of
#
# Output:
# the parent directory name of the given path
k_fs_dirname() {
	# shellcheck disable=SC2039
	local path dirPath

	k_requires_args k_fs_dirname $#

	path="${1%/}"
	dirPath="${path%/*}"

	if [ -n "${path}" ] && [ "${path}" = "${dirPath}" ]; then
		dirPath="."
	fi

	printf -- '%s\n' "${dirPath:-/}"
}

# Get a temp directory
#
# Output:
# the temp directory path
k_fs_temp_dir() {
	# shellcheck disable=SC2039
	local tmpDir

	tmpDir="$(mktemp -d)"

	k_log_debug 'Created temporary directory: "%s"' "${tmpDir}"

	k_fs_register_temp_file "${tmpDir}"

	printf -- '%s\n' "${tmpDir}"
}

# Get a temp file
#
# Output:
# the temp file path
k_fs_temp_file() {
	# shellcheck disable=SC2039
	local tmpFile

	tmpFile="$(mktemp)"

	k_log_debug 'Created temporary file: "%s"' "${tmpFile}"

	k_fs_register_temp_file "${tmpFile}"

	printf -- '%s\n' "${tmpFile}"
}

# Register a file as temporary, for automatic cleanup
#
# Input:
# $1...n the filename(s) to register as temporary files
k_fs_register_temp_file() {
	printf -- '%s\n' "$@" >> "$(k_fs_predictable_file "${K_FS_TEMP_FILES_FILENAME}")"
}

# Get a 'predictable' temporary directory
# The directory path is generated using constants for this process, so they can be retrieved within a trap callback
#
# Input:
# $1 - the basename for the directory. If not specified, the output of (#k_tool_name) is used
# $2 - the optional suffix for the directory
#
# Output:
# the temporary directory name
# Return:
# 0 if the directory is usable, 1 if it cannot be used due to a file existing with the same name.
k_fs_predictable_dir() {
	# shellcheck disable=SC2039
	local name suffix tmpDir

	name="${1:-$(k_tool_name)}"
	suffix="${2:-}"
	tmpDir="${TMPDIR:-/tmp}/${name}.${$}${suffix}"

	if ! mkdir -p "${tmpDir}"; then
		if [ -f "${tmpDir}" ]; then
			k_log_err 'Cannot create predictable directory, file exists already: "%s"' "${tmpDir}"
			return 1
		fi
	fi

	k_log_debug 'Using predictable temporary directory: "%s"' "${tmpDir}"

	k_fs_register_temp_file "${tmpDir}"

	printf -- '%s\n' "${tmpDir}"
}

# Get a 'predictable' temporary file
# The file path is generated using constants for this process, so they can be retrieved within a trap callback
#
# Input:
# $1 - the basename for the file. If not specified, the output of (#k_tool_name) is used
# $2 - the optional suffix for the filename
#
# Output:
# the temporary file name
#
# Return:
# 0 if the file is usable, 1 if it cannot be used due to a directory existing with the same name.
k_fs_predictable_file() {
	# shellcheck disable=SC2039
	local name suffix tmpFile initialFormat=''

	name="${1:-$(k_tool_name)}"
	suffix="${2:-}"
	tmpFile="${TMPDIR:-/tmp}/${name}.${$}${suffix}"

	if [ "${name}" = "${K_FS_TEMP_FILES_FILENAME}" ] && [ -z "${suffix}" ]; then
		initialFormat='%s\n'
	fi

	k_log_debug 'Using predictable temporary file: "%s"' "${tmpFile}"

	# shellcheck disable=SC2059
	if ! (set -C; printf -- "${initialFormat}" "${tmpFile}" 2>/dev/null > "${tmpFile}"); then
		if [ -d "${tmpFile}" ]; then
			k_log_err 'Cannot create predictable file, directory exists already: "%s"' "${tmpFile}"
			return 1
		else
			k_log_debug 'Using existing predictable temp file "%s"' "${tmpFile}"
		fi
	fi

	if [ -z "${initialFormat}" ]; then
		k_fs_register_temp_file "${tmpFile}"
	fi

	printf -- '%s\n' "${tmpFile}"
}

# Cleanup temp files and directories
k_fs_temp_cleanup() {
	# shellcheck disable=SC2039
	local tempFilesFile

	tempFilesFile="$(k_fs_predictable_file "${K_FS_TEMP_FILES_FILENAME}")"
	k_log_info "Cleaning up temp files"

	while IFS= read -r file; do
		k_log_debug 'Removing "%s"' "${file}"
		rm -rf "${file}"
	done < "${tempFilesFile}"
}

# Make a file/directory accessible only by the named or current user
# Changes user/group ownership and sets read/write (+executable if a directory or already executable) permission to user only
#
# Input:
# $1 - the path to set ownership/permissions on
# $2 - the username to set ownership to. If omitted, uses the current user
# $3 - the group to set ownership to. If omitted, uses the current user's primary group
k_fs_perms_useronly() {
	# shellcheck disable=SC2039
	local path user group

	k_requires_args k_fs_perms_useronly $#

	path="${1}"
	user="${2:-$(id -n -u)}"
	group="${3:-$(id -n -g)}"

	k_log_debug 'Setting ownership of "%s"' "${path}"
	chown -R "${user}:${group}" "${path}"

	k_log_debug 'Setting permissions on "%s"' "${path}"
	chmod -R u=rwX,go= "${path}"
}

# Run (#k_fs_temp_cleanup) when the program exits.
# This uses trap to listen for the signals INT TERM QUIT EXIT HUP
# This should handle most causes of exit:
# Regular program close, Ctrl-C, SIGHUP, etc.
# Note: This does NOT cleanup temp files if the program receives the ABRT signal.
# Use (#k_fs_temp_abort) to cleanup on ABRT.
k_fs_temp_exit() {
	cleanup_exit() {
		k_log_debug "Caught exit Signal"
		trap - INT TERM QUIT EXIT HUP
		k_fs_temp_cleanup
	}

	trap cleanup_exit INT TERM QUIT EXIT HUP
}

# Run (#k_fs_temp_cleanup) when the program is killed.
# This uses trap to listen for the signal ABRT
# Note: This will ONLY cleanup temp files if the program receives the ABRT signal.
# Use (#k_fs_temp_exit) to cleanup on INT TERM QUIT EXIT HUP.
k_fs_temp_abort() {
	cleanup_abort() {
		k_log_debug "Caught abort Signal"
		trap - ABRT
		k_fs_temp_cleanup
	}

	trap cleanup_abort ABRT
}
